#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2018 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_dlp_fp_doc_source
short_description: Create a DLP fingerprint database by allowing the FortiGate to access a file server containing files from which to create fingerprints in
   Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by
      allowing the user to configure dlp feature and fp_doc_source category.
      Examples includes all options and need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: false
    dlp_fp_doc_source:
        description:
            - Create a DLP fingerprint database by allowing the FortiGate to access a file server containing files from which to create fingerprints.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            date:
                description:
                    - Day of the month on which to scan the server (1 - 31).
            file-path:
                description:
                    - Path on the server to the fingerprint files (max 119 characters).
            file-pattern:
                description:
                    - Files matching this pattern on the server are fingerprinted. Optionally use the * and ? wildcards.
            keep-modified:
                description:
                    - Enable so that when a file is changed on the server the FortiGate keeps the old fingerprint and adds a new fingerprint to the database.
                choices:
                    - enable
                    - disable
            name:
                description:
                    - Name of the DLP fingerprint database.
                required: true
            password:
                description:
                    - Password required to log into the file server.
            period:
                description:
                    - Frequency for which the FortiGate checks the server for new or changed files.
                choices:
                    - none
                    - daily
                    - weekly
                    - monthly
            remove-deleted:
                description:
                    - Enable to keep the fingerprint database up to date when a file is deleted from the server.
                choices:
                    - enable
                    - disable
            scan-on-creation:
                description:
                    - Enable to keep the fingerprint database up to date when a file is added or changed on the server.
                choices:
                    - enable
                    - disable
            scan-subdirectories:
                description:
                    - Enable/disable scanning subdirectories to find files to create fingerprints from.
                choices:
                    - enable
                    - disable
            sensitivity:
                description:
                    - Select a sensitivity or threat level for matches with this fingerprint database. Add sensitivities using fp-sensitivity. Source dlp
                      .fp-sensitivity.name.
            server:
                description:
                    - IPv4 or IPv6 address of the server.
            server-type:
                description:
                    - Protocol used to communicate with the file server. Currently only Samba (SMB) servers are supported.
                choices:
                    - samba
            tod-hour:
                description:
                    - Hour of the day on which to scan the server (0 - 23, default = 1).
            tod-min:
                description:
                    - Minute of the hour on which to scan the server (0 - 59).
            username:
                description:
                    - User name required to log into the file server.
            vdom:
                description:
                    - Select the VDOM that can communicate with the file server.
                choices:
                    - mgmt
                    - current
            weekday:
                description:
                    - Day of the week on which to scan the server.
                choices:
                    - sunday
                    - monday
                    - tuesday
                    - wednesday
                    - thursday
                    - friday
                    - saturday
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Create a DLP fingerprint database by allowing the FortiGate to access a file server containing files from which to create fingerprints.
    fortios_dlp_fp_doc_source:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      dlp_fp_doc_source:
        state: "present"
        date: "3"
        file-path: "<your_own_value>"
        file-pattern: "<your_own_value>"
        keep-modified: "enable"
        name: "default_name_7"
        password: "<your_own_value>"
        period: "none"
        remove-deleted: "enable"
        scan-on-creation: "enable"
        scan-subdirectories: "enable"
        sensitivity: "<your_own_value> (source dlp.fp-sensitivity.name)"
        server: "192.168.100.40"
        server-type: "samba"
        tod-hour: "16"
        tod-min: "17"
        username: "<your_own_value>"
        vdom: "mgmt"
        weekday: "sunday"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_dlp_fp_doc_source_data(json):
    option_list = ['date', 'file-path', 'file-pattern',
                   'keep-modified', 'name', 'password',
                   'period', 'remove-deleted', 'scan-on-creation',
                   'scan-subdirectories', 'sensitivity', 'server',
                   'server-type', 'tod-hour', 'tod-min',
                   'username', 'vdom', 'weekday']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def dlp_fp_doc_source(data, fos):
    vdom = data['vdom']
    dlp_fp_doc_source_data = data['dlp_fp_doc_source']
    filtered_data = filter_dlp_fp_doc_source_data(dlp_fp_doc_source_data)
    if dlp_fp_doc_source_data['state'] == "present":
        return fos.set('dlp',
                       'fp-doc-source',
                       data=filtered_data,
                       vdom=vdom)

    elif dlp_fp_doc_source_data['state'] == "absent":
        return fos.delete('dlp',
                          'fp-doc-source',
                          mkey=filtered_data['name'],
                          vdom=vdom)


def fortios_dlp(data, fos):
    login(data)

    methodlist = ['dlp_fp_doc_source']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": "False"},
        "dlp_fp_doc_source": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "date": {"required": False, "type": "int"},
                "file-path": {"required": False, "type": "str"},
                "file-pattern": {"required": False, "type": "str"},
                "keep-modified": {"required": False, "type": "str",
                                  "choices": ["enable", "disable"]},
                "name": {"required": True, "type": "str"},
                "password": {"required": False, "type": "str"},
                "period": {"required": False, "type": "str",
                           "choices": ["none", "daily", "weekly",
                                       "monthly"]},
                "remove-deleted": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "scan-on-creation": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "scan-subdirectories": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                "sensitivity": {"required": False, "type": "str"},
                "server": {"required": False, "type": "str"},
                "server-type": {"required": False, "type": "str",
                                "choices": ["samba"]},
                "tod-hour": {"required": False, "type": "int"},
                "tod-min": {"required": False, "type": "int"},
                "username": {"required": False, "type": "str"},
                "vdom": {"required": False, "type": "str",
                         "choices": ["mgmt", "current"]},
                "weekday": {"required": False, "type": "str",
                            "choices": ["sunday", "monday", "tuesday",
                                        "wednesday", "thursday", "friday",
                                        "saturday"]}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_dlp(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



from __future__ import absolute_import

import pytest
from ansible.module_utils import basic
from units.modules.utils import set_module_args, exit_json, fail_json, AnsibleFailJson, AnsibleExitJson

from ansible.modules.network.ftd import ftd_file_upload
from ansible.module_utils.network.ftd.fdm_swagger_client import OperationField
from ansible.module_utils.network.ftd.common import HTTPMethod


class TestFtdFileUpload(object):
    module = ftd_file_upload

    @pytest.fixture(autouse=True)
    def module_mock(self, mocker):
        return mocker.patch.multiple(basic.AnsibleModule, exit_json=exit_json, fail_json=fail_json)

    @pytest.fixture
    def connection_mock(self, mocker):
        connection_class_mock = mocker.patch('ansible.modules.network.ftd.ftd_file_upload.Connection')
        return connection_class_mock.return_value

    @pytest.mark.parametrize("missing_arg", ['operation', 'file_to_upload'])
    def test_module_should_fail_without_required_args(self, missing_arg):
        module_args = {'operation': 'uploadFile', 'file_to_upload': '/tmp/test.txt'}
        del module_args[missing_arg]
        set_module_args(module_args)

        with pytest.raises(AnsibleFailJson) as ex:
            self.module.main()

        assert 'missing required arguments: %s' % missing_arg in str(ex.value)

    def test_module_should_fail_when_no_operation_spec_found(self, connection_mock):
        connection_mock.get_operation_spec.return_value = None
        set_module_args({'operation': 'nonExistingUploadOperation', 'file_to_upload': '/tmp/test.txt'})

        with pytest.raises(AnsibleFailJson) as ex:
            self.module.main()

        result = ex.value.args[0]
        assert result['failed']
        assert result['msg'] == 'Operation with specified name is not found: nonExistingUploadOperation'

    def test_module_should_fail_when_not_upload_operation_specified(self, connection_mock):
        connection_mock.get_operation_spec.return_value = {
            OperationField.METHOD: HTTPMethod.GET,
            OperationField.URL: '/object/network',
            OperationField.MODEL_NAME: 'NetworkObject'
        }
        set_module_args({'operation': 'nonUploadOperation', 'file_to_upload': '/tmp/test.txt'})

        with pytest.raises(AnsibleFailJson) as ex:
            self.module.main()

        result = ex.value.args[0]
        assert result['failed']
        assert result['msg'] == 'Invalid upload operation: nonUploadOperation. ' \
                                'The operation must make POST request and return UploadStatus model.'

    def test_module_should_call_upload_and_return_response(self, connection_mock):
        connection_mock.get_operation_spec.return_value = {
            OperationField.METHOD: HTTPMethod.POST,
            OperationField.URL: '/uploadFile',
            OperationField.MODEL_NAME: 'FileUploadStatus'
        }
        connection_mock.upload_file.return_value = {'id': '123'}

        set_module_args({
            'operation': 'uploadFile',
            'file_to_upload': '/tmp/test.txt'
        })
        with pytest.raises(AnsibleExitJson) as ex:
            self.module.main()

        result = ex.value.args[0]
        assert result['changed']
        assert {'id': '123'} == result['response']
        connection_mock.upload_file.assert_called_once_with('/tmp/test.txt', '/uploadFile')



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2018 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_webfilter_fortiguard
short_description: Configure FortiGuard Web Filter service.
description:
    - This module is able to configure a FortiGate or FortiOS by
      allowing the user to configure webfilter feature and fortiguard category.
      Examples includes all options and need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: false
    webfilter_fortiguard:
        description:
            - Configure FortiGuard Web Filter service.
        default: null
        suboptions:
            cache-mem-percent:
                description:
                    - Maximum percentage of available memory allocated to caching (1 - 15%).
            cache-mode:
                description:
                    - Cache entry expiration mode.
                choices:
                    - ttl
                    - db-ver
            cache-prefix-match:
                description:
                    - Enable/disable prefix matching in the cache.
                choices:
                    - enable
                    - disable
            close-ports:
                description:
                    - Close ports used for HTTP/HTTPS override authentication and disable user overrides.
                choices:
                    - enable
                    - disable
            ovrd-auth-https:
                description:
                    - Enable/disable use of HTTPS for override authentication.
                choices:
                    - enable
                    - disable
            ovrd-auth-port:
                description:
                    - Port to use for FortiGuard Web Filter override authentication.
            ovrd-auth-port-http:
                description:
                    - Port to use for FortiGuard Web Filter HTTP override authentication
            ovrd-auth-port-https:
                description:
                    - Port to use for FortiGuard Web Filter HTTPS override authentication.
            ovrd-auth-port-warning:
                description:
                    - Port to use for FortiGuard Web Filter Warning override authentication.
            request-packet-size-limit:
                description:
                    - Limit size of URL request packets sent to FortiGuard server (0 for default).
            warn-auth-https:
                description:
                    - Enable/disable use of HTTPS for warning and authentication.
                choices:
                    - enable
                    - disable
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure FortiGuard Web Filter service.
    fortios_webfilter_fortiguard:
      host:  "{{  host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{  vdom }}"
      webfilter_fortiguard:
        cache-mem-percent: "3"
        cache-mode: "ttl"
        cache-prefix-match: "enable"
        close-ports: "enable"
        ovrd-auth-https: "enable"
        ovrd-auth-port: "8"
        ovrd-auth-port-http: "9"
        ovrd-auth-port-https: "10"
        ovrd-auth-port-warning: "11"
        request-packet-size-limit: "12"
        warn-auth-https: "enable"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "key1"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_webfilter_fortiguard_data(json):
    option_list = ['cache-mem-percent', 'cache-mode', 'cache-prefix-match',
                   'close-ports', 'ovrd-auth-https', 'ovrd-auth-port',
                   'ovrd-auth-port-http', 'ovrd-auth-port-https', 'ovrd-auth-port-warning',
                   'request-packet-size-limit', 'warn-auth-https']
    dictionary = {}

    for attribute in option_list:
        if attribute in json:
            dictionary[attribute] = json[attribute]

    return dictionary


def webfilter_fortiguard(data, fos):
    vdom = data['vdom']
    webfilter_fortiguard_data = data['webfilter_fortiguard']
    filtered_data = filter_webfilter_fortiguard_data(webfilter_fortiguard_data)
    return fos.set('webfilter',
                   'fortiguard',
                   data=filtered_data,
                   vdom=vdom)


def fortios_webfilter(data, fos):
    host = data['host']
    username = data['username']
    password = data['password']
    fos.https('off')
    fos.login(host, username, password)

    methodlist = ['webfilter_fortiguard']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": "False"},
        "webfilter_fortiguard": {
            "required": False, "type": "dict",
            "options": {
                "cache-mem-percent": {"required": False, "type": "int"},
                "cache-mode": {"required": False, "type": "str",
                               "choices": ["ttl", "db-ver"]},
                "cache-prefix-match": {"required": False, "type": "str",
                                       "choices": ["enable", "disable"]},
                "close-ports": {"required": False, "type": "str",
                                "choices": ["enable", "disable"]},
                "ovrd-auth-https": {"required": False, "type": "str",
                                    "choices": ["enable", "disable"]},
                "ovrd-auth-port": {"required": False, "type": "int"},
                "ovrd-auth-port-http": {"required": False, "type": "int"},
                "ovrd-auth-port-https": {"required": False, "type": "int"},
                "ovrd-auth-port-warning": {"required": False, "type": "int"},
                "request-packet-size-limit": {"required": False, "type": "int"},
                "warn-auth-https": {"required": False, "type": "str",
                                    "choices": ["enable", "disable"]}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_webfilter(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
#
# Copyright: (c) 2017, F5 Networks Inc.
# GNU General Public License v3.0 (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import os
import json
import pytest
import sys

if sys.version_info < (2, 7):
    pytestmark = pytest.mark.skip("F5 Ansible modules require Python >= 2.7")

from ansible.module_utils.basic import AnsibleModule

try:
    from library.modules.bigip_gtm_monitor_https import ApiParameters
    from library.modules.bigip_gtm_monitor_https import ModuleParameters
    from library.modules.bigip_gtm_monitor_https import ModuleManager
    from library.modules.bigip_gtm_monitor_https import ArgumentSpec

    # In Ansible 2.8, Ansible changed import paths.
    from test.units.compat import unittest
    from test.units.compat.mock import Mock
    from test.units.compat.mock import patch

    from test.units.modules.utils import set_module_args
except ImportError:
    from ansible.modules.network.f5.bigip_gtm_monitor_https import ApiParameters
    from ansible.modules.network.f5.bigip_gtm_monitor_https import ModuleParameters
    from ansible.modules.network.f5.bigip_gtm_monitor_https import ModuleManager
    from ansible.modules.network.f5.bigip_gtm_monitor_https import ArgumentSpec

    # Ansible 2.8 imports
    from units.compat import unittest
    from units.compat.mock import Mock
    from units.compat.mock import patch

    from units.modules.utils import set_module_args


fixture_path = os.path.join(os.path.dirname(__file__), 'fixtures')
fixture_data = {}


def load_fixture(name):
    path = os.path.join(fixture_path, name)

    if path in fixture_data:
        return fixture_data[path]

    with open(path) as f:
        data = f.read()

    try:
        data = json.loads(data)
    except Exception:
        pass

    fixture_data[path] = data
    return data


class TestParameters(unittest.TestCase):
    def test_module_parameters(self):
        args = dict(
            name='foo',
            parent='/Common/my-http',
            send='the send string',
            receive='the receive string',
            ip='1.1.1.1',
            port='80',
            interval='10',
            timeout='20',
            client_cert='default',
            client_key='default',
            target_username='user1',
            target_password='secret1',
            ignore_down_response=True,
            transparent=False,
            probe_timeout='30',
            reverse=True
        )

        p = ModuleParameters(params=args)
        assert p.name == 'foo'
        assert p.parent == '/Common/my-http'
        assert p.send == 'the send string'
        assert p.receive == 'the receive string'
        assert p.destination == '1.1.1.1:80'
        assert p.ip == '1.1.1.1'
        assert p.port == 80
        assert p.interval == 10
        assert p.timeout == 20
        assert p.client_cert == '/Common/default.crt'
        assert p.client_key == '/Common/default.key'
        assert p.target_username == 'user1'
        assert p.target_password == 'secret1'
        assert p.ignore_down_response is True
        assert p.transparent is False
        assert p.probe_timeout == 30
        assert p.reverse is True

    def test_api_parameters(self):
        args = load_fixture('load_gtm_monitor_http_1.json')

        p = ApiParameters(params=args)
        assert p.name == 'foo'
        assert p.parent == '/Common/http'
        assert p.send == 'GET /'
        assert p.receive == 'the receive string'
        assert p.destination == '3.3.3.3:8080'
        assert p.ip == '3.3.3.3'
        assert p.port == 8080
        assert p.interval == 30
        assert p.timeout == 120
        assert p.ignore_down_response is False
        assert p.transparent is True
        assert p.probe_timeout == 5
        assert p.reverse is True


class TestManager(unittest.TestCase):

    def setUp(self):
        self.spec = ArgumentSpec()

        try:
            self.p1 = patch('library.modules.bigip_gtm_monitor_https.module_provisioned')
            self.m1 = self.p1.start()
            self.m1.return_value = True
        except Exception:
            self.p1 = patch('ansible.modules.network.f5.bigip_gtm_monitor_https.module_provisioned')
            self.m1 = self.p1.start()
            self.m1.return_value = True

    def tearDown(self):
        self.p1.stop()

    def test_create_monitor(self, *args):
        set_module_args(dict(
            name='foo',
            ip='10.10.10.10',
            port=80,
            interval=20,
            timeout=30,
            provider=dict(
                server='localhost',
                password='password',
                user='admin'
            )
        ))

        module = AnsibleModule(
            argument_spec=self.spec.argument_spec,
            supports_check_mode=self.spec.supports_check_mode
        )

        # Override methods in the specific type of manager
        mm = ModuleManager(module=module)
        mm.exists = Mock(side_effect=[False, True])
        mm.create_on_device = Mock(return_value=True)
        mm.module_provisioned = Mock(return_value=True)

        results = mm.exec_module()

        assert results['changed'] is True
